<?php
/*
=====================================================
 AJAX Pagination
 by: Cody Lundquist
 http://www.codysplace.com
 Copyright (c) Cody Lundquist
=============================================================
	 THIS IS COPYRIGHTED SOFTWARE
-------------------------------------------------------------
	This software is based upon and derived from
	EllisLab ExpressionEngine software protected under
	copyright dated 2004 - 2007. Please see
	www.expressionengine.com/docs/license.html
=============================================================
	File:			pi.ajax_pagination.php
-------------------------------------------------------------
	Compatibility:	ExpressionEngine 1.6+ 
	                jQuery 1.2.6+
-------------------------------------------------------------
	Purpose:		Use AJAX to Paginate Entries
=============================================================
*/


$plugin_info = array(
						'pi_name'			=> 'AJAX Pagination',
						'pi_version'		=> '1.3',
						'pi_author'			=> 'MeanStudios',
						'pi_author_url'		=> 'http://meanstudios.com/',
						'pi_description'	=> 'Uses AJAX to paginate entries without refreshing page.',
						'pi_usage'			=> Ajax_pagination::usage()
					);


class Ajax_pagination {

    var $return_data;

    
    /** ----------------------------------------
    /**  AJAX Pagination
    /** ----------------------------------------*/

	function Ajax_pagination()
	{
		global $TMPL, $DB, $IN, $FNS;

		/* Grab Template Data */
		$tagdata = $TMPL->tagdata;
		
		/* Get tag parameters */
		$limit = ( ! $TMPL->fetch_param('limit')) ? '4' : $TMPL->fetch_param('limit');
		$weblog = ( ! $TMPL->fetch_param('weblog')) ? '' : $TMPL->fetch_param('weblog');
		$template = ( ! $TMPL->fetch_param('template')) ? '' : $TMPL->fetch_param('template');
		$holder = ( ! $TMPL->fetch_param('holder')) ? '' : $TMPL->fetch_param('holder');
		$parse = ( ! $TMPL->fetch_param('parse')) ? '' : $TMPL->fetch_param('holder');
		$next = ( ! $TMPL->fetch_param('next')) ? 'Next &gt;&gt;' : $TMPL->fetch_param('next');
		$prev = ( ! $TMPL->fetch_param('prev')) ? '&lt;&lt; Prev' : $TMPL->fetch_param('prev');
		$next_class = ( ! $TMPL->fetch_param('next_class')) ? '' : $TMPL->fetch_param('next_class');
		$prev_class = ( ! $TMPL->fetch_param('prev_class')) ? '' : $TMPL->fetch_param('prev_class');
		
		/* Setup output variables */
		$prev_link = "";
		$next_link = "";
		
		/* Setup Offset variables */
		preg_match('/^([0-9]+)\/([0-9]+)/', $IN->QSTR, $matches);
		$future_offset = (isset($matches[2])) ? $matches[2] : $limit;
		$weblog_offset = (isset($matches[1])) ? $matches[1] : "";
		
		/* Check to make sure the required parameters are set */
		if( ! $weblog OR ! $template OR ! $holder OR ! $parse)
		{
			$this->return_data = "You have left at least one of the required parameters blank in the AJAX Pagination plugin tag.  Please correct your syntax.";
			return $this->return_data;
		}
		
		/* Query to check if the Next link is needed */
		$query = $DB->query("SELECT a.weblog_id, b.title
							 FROM exp_weblogs a, exp_weblog_titles b
							 WHERE a.blog_name = '" . $weblog . "' 
							 AND a.weblog_id = b.weblog_id
							 LIMIT 1
							 OFFSET " . $future_offset);
		
		/* Build the Prev link */
		if (isset($IN->SEGS[3]))
		{ 
			$prev_link = "<a style=\"cursor: pointer;\" rel=\"";
			if ($IN->SEGS[3] > $limit)
			{
				$prev_link .= ($IN->SEGS[3] - $limit) . "/0";
			}
			$prev_link .= "\" class=\"" . $prev_class . "\" id=\"previous\">" . $prev . "</a>";
		}
		
		/* Build the Next link */
		if (isset($query->row['title']))
		{
			$next_link = "<a style=\"cursor: pointer;\" rel=\"";
			if (isset($IN->SEGS[3]))
			{
				$next_link .= ($IN->SEGS[3] + $limit) . "/" . ($IN->SEGS[3] + ($limit * 2)); 
			} else { 
				$next_link .= ($limit) . "/" . ($limit * 2);
			}
			$next_link .= "\" class=\"" . $next_class . "\" id=\"next\">" . $next . "</a>";
		}
		
		/* Pump out jQuery Magic */
		ob_start();
?>
		<script type="text/javascript">
		$(function() {
				$('#next').click(function(){
					$("#<?= $holder ?>").load("<?= $template ?>/" + $("#next").attr("rel"));
					return false;
				});
				$('#previous').click(function(){
					$("#<?= $holder ?>").load("<?= $template ?>/" + $("#previous").attr("rel"));
					return false;
				});
		});
		</script>
<?php
		$javascript = ob_get_contents();
		ob_end_clean();
		
		/* Setup the two conditionals */
		$cond['prev_link']		= ($prev_link == "") ? 'FALSE' : 'TRUE';
		$cond['next_link']		= ($next_link == "") ? 'FALSE' : 'TRUE';
		
		$tagdata = $FNS->prep_conditionals($tagdata, $cond);

		/* Check to make sure all variables for this plugin are being used */
		if ( ! array_key_exists('prev_link', $TMPL->var_single) OR ! array_key_exists('next_link', $TMPL->var_single) OR ! array_key_exists('offset', $TMPL->var_single) OR ! array_key_exists('limit', $TMPL->var_single))
		{
			$this->return_data = "One or more variables are not being used in the AJAX Pagination plugin.  All variables are required for this plugin to work.  Please correct your syntax.";
			return $this->return_data;
		}
		
		/* Output the complied data to the template */
		foreach ($TMPL->var_single as $key => $val)
		{
			if ($val == "prev_link")
			{
				$tagdata = $TMPL->swap_var_single($val, $prev_link, $tagdata);
			}
			if ($val == "next_link")
			{
				$tagdata = $TMPL->swap_var_single($val, $next_link, $tagdata);
			}
			if ($val == "offset")
			{
				$tagdata = $TMPL->swap_var_single($val, $weblog_offset, $tagdata);
			}
			if ($val == "limit")
			{
				$tagdata = $TMPL->swap_var_single($val, $limit, $tagdata);
			}
		}
		$this->return_data = $tagdata . $javascript;
	}
    /* END */
    
// ----------------------------------------
//  Plugin Usage
// ----------------------------------------

// This function describes how the plugin is used.
//  Make sure and use output buffering

function usage()
{
ob_start(); 
?>
 :: Setup :: 
1) Create your main template where your entries will be shown.  This is where you Holder <div> needs to be located and inside that Holder <div> you need to embed the AJAX template. Also jQuery 1.2.6 needs to be included in the header tags.  An example will be shown below.
2) Create your AJAX template for the ajaxification (real word?) to take place.  Make sure in your {exp:weblog:entries} tag you include limit="{limit}", offset="{offset}" and dynamic="off". Again, example will be shown below.

:: Parameters ::
These parameters must not be blank:
weblog - Define which weblog you are pulling entries from.
template - The full url to your Ajax Template.
holder - Define what <div> on your main template is going to hold the entries.
parse - This needs to be set to "inward" for the plugin to work.

These parameters are optional:
limit - Define how many entries should be shown per page. Default is 4 if left blank.
next - Define exactly how you want the "Next" text to look.  Default is "Next >>" if left blank.
prev - Define exactly how you want the "Prev" text to look.  Default is "<< Prev" if left blank.
next_class - Define the CSS class you want to use for the Next link. Default is class="" if left blank.
prev_class - Define the CSS class you want to use for the Prev link. Default is class="" if left blank.

:: Variables ::
The plugin will not work if any of these variables are missing in your template.
{prev_link} - This gets replaced by the Previous Link. Can also be used as conditional. Example below.
{next_link} - This gets replaced by the Next Link. Can also be used as conditional. Example below.
{limit} - This tells the {exp:weblog:entries} tag how many to show each page as told by the "limit=" parameter above.
{offset} - This tells the {exp:weblog:entries} tag how many entries to offset.

:: Barebones Example Usage :: 
Main Template:
<html>
<head>
<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.2.6/jquery.min.js"></script>
</head>
<body>
<div id="wrapper">
<div id="body">
    <h1>This is an example for AJAX Pagination</h1>
    <div id="ajax_content">{embed="template_group/ajax_template"}</div>
  </div>
</div>
</body>
</html>

AJAX Template:
{exp:ajax_pagination weblog="weblog" limit="4" template="{site_url}template_group/ajax_template" holder="ajax_content" parse="inward"}
{prev_link} {if prev_link && next_link}|{/if} {next_link}
{exp:weblog:entries weblog="weblog" disable="pagination|member_data|trackbacks" limit="{limit}" offset="{offset}" dynamic="off"}
<h3>{title}</h3>
{body}
{/exp:weblog:entries}
{/exp:ajax_pagination}

<?php
$buffer = ob_get_contents();
	
ob_end_clean(); 

return $buffer;
}
/* END */


}
// END CLASS
?>